@@ -5,37 +5,22 @@
 # import asyncio
 import requests
 
-# set global variables
+# declare global variables
+global HEADERS
 global models
 global incantations
-global headers
-global base_url
 
-# initialize global variables
-models = None
-incantations = None
-headers = None
-base_url = 'https://58shnvgnlf.execute-api.us-east-1.amazonaws.com/sandbox/'
+HEADERS = None
+models = {}
+incantations = {}
 
 
-# exceptions
-class NoApiKeyError(Exception):
-    def __init__(self):
-        super().__init__('No API key has been set. Please run check that your API key is correct and then run `superpowered.init(api_key)` before continuing')
-
-
-def init(api_key: str):
-    api_key = {'x-api-key': api_key}
-
-
-def get_current_state():
-    models = get_models()
-    incantations = get_incantations()
+BASE_URL = 'https://58shnvgnlf.execute-api.us-east-1.amazonaws.com/sandbox/'
 
 
 def get_models():
-    url = base_url + 'models'
-    resp = requests.get(url, headers=headers)
+    url = BASE_URL + 'models'
+    resp = requests.get(url, headers=HEADERS)
     resp = _format_http_response(resp)
     for model in resp['body']:
         model = Model(
@@ -48,8 +33,8 @@ def get_models():
 
 
 def get_incantations():
-    url = base_url + 'incantations'
-    resp = requests.get(url, headers=headers)
+    url = BASE_URL + 'incantations'
+    resp = requests.get(url, headers=HEADERS)
     resp = _format_http_response(resp)
     for incantation in resp['body']:
         incantation = Incantation(
@@ -62,6 +47,26 @@ def get_incantations():
         incantations[incantation['id']] = incantation
 
 
+def init(api_key: str):
+    global HEADERS
+    global models
+    global incantations
+    HEADERS = {'x-api-key': api_key}
+
+    # get the current state with models and incantations
+    models = get_models()
+    incantations = get_incantations()
+
+    print(models)
+    print(incantations)
+
+
+# exceptions
+class NoApiKeyError(Exception):
+    def __init__(self):
+        super().__init__('No API key has been set. Please run check that your API key is correct and then run `superpowered.init(api_key)` before continuing')
+
+
 def _format_http_response(resp: requests.Response):
     if resp.status_code != 200:
         raise Exception(resp.json())
@@ -83,14 +88,14 @@ class Incantation:
     def create(self):
         if self.is_deployed:
             raise Exception('This incantation has already been deployed: ' + self.incantation_id)
-        url = self.base_url + 'incantations'
+        url = self.BASE_URL + 'incantations'
         payload = {
             'title': self.title,
             'text': self.text,
             'supp_id': self.supp_id,
             'description': self.description
         }
-        resp = _format_http_response(requests.post(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.post(url, headers=HEADERS, json=payload))
         self.incantation_id = self.get_incantation_id()
         self.is_deployed = True
         incantations[resp['body']['id']]
@@ -99,7 +104,7 @@ class Incantation:
     def update(self):
         if not self.is_deployed:
             raise Exception('This incantation has not been deployed yet. Please run `incantation.create()` before running `incantation.update()')
-        url = self.base_url + f'incantations/{self.incantation_id}'
+        url = self.BASE_URL + f'incantations/{self.incantation_id}'
         payload = {}
         if self.title is not None:
             payload['title'] = self.title
@@ -109,12 +114,12 @@ class Incantation:
             payload['supp_id'] = self.supp_id
         if self.description is not None:
             payload['description'] = self.description
-        resp = _format_http_response(requests.put(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.put(url, headers=HEADERS, json=payload))
         return resp['body']
 
     def delete(self, incantation_id: str):
-        url = self.base_url + f'incantations/{incantation_id}'
-        _format_http_response(requests.delete(url, headers=self.headers))
+        url = self.BASE_URL + f'incantations/{incantation_id}'
+        _format_http_response(requests.delete(url, headers=HEADERS))
         del incantations[incantation_id]
 
 
@@ -130,13 +135,13 @@ class Model:
     def create(self):
         if self.is_deployed:
             raise Exception('This model has already been deployed: ' + self.model_id)
-        url = self.base_url + 'models'
+        url = self.BASE_URL + 'models'
         payload = {
             'model_spec': self.model_spec,
             'supp_id': self.supp_id,
             'description': self.description
         }
-        resp = _format_http_response(requests.post(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.post(url, headers=HEADERS, json=payload))
         self.model_id = self.get_model_id()
         self.is_deployed = True
         models[resp['body']['id']]
@@ -145,7 +150,7 @@ class Model:
     def update(self):
         if not self.is_deployed:
             raise Exception('This model has not been deployed yet. Please run `model.create()` before running `model.update()')
-        url = self.base_url + f'models/{self.model_id}'
+        url = self.BASE_URL + f'models/{self.model_id}'
         payload = {}
         if self.model_spec is not None:
             payload['model_spec'] = self.model_spec
@@ -153,18 +158,18 @@ class Model:
             payload['supp_id'] = self.supp_id
         if self.description is not None:
             payload['description'] = self.description
-        resp = _format_http_response(requests.put(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.put(url, headers=HEADERS, json=payload))
         return resp['body']
 
     def delete(self):
-        url = self.base_url + f'models/{self.model_id}'
-        _format_http_response(requests.delete(url, headers=self.headers))
+        url = self.BASE_URL + f'models/{self.model_id}'
+        _format_http_response(requests.delete(url, headers=HEADERS))
         del models[self.model_id]
         return True
 
     def get_instances(self):
-        url = self.base_url + f'models/{self.model_id}/instances'
-        resp = _format_http_response(requests.get(url, headers=self.headers))
+        url = self.BASE_URL + f'models/{self.model_id}/instances'
+        resp = _format_http_response(requests.get(url, headers=HEADERS))
         instances = {}
         for instance in resp['body']:
             instance = ModelInstance(
@@ -188,22 +193,22 @@ class ModelInstance:
 
     def get_chat_history(self):
         # get chat history interactions with pagination
-        url = self.base_url + f'models/{self.model_id}/instances/{self.instance_id}/chat_history'
-        response = _format_http_response(requests.get(url, headers=self.headers))
+        url = self.BASE_URL + f'models/{self.model_id}/instances/{self.instance_id}/chat_history'
+        response = _format_http_response(requests.get(url, headers=HEADERS))
         self.chat_history.extend(response['body']['interactions'])
         while 'next_page_token' in response:
-            response = _format_http_response(requests.get(url, headers=self.headers, params={'next_page_token': response['next_page_token']}))
+            response = _format_http_response(requests.get(url, headers=HEADERS, params={'next_page_token': response['next_page_token']}))
             self.chat_history.extend(response['body']['interactions'])
 
     def create(self):
         if self.is_deployed:
             raise Exception('This model instance has already been deployed: ' + self.instance_id)
-        url = self.base_url + f'models/{self.model_id}/instances'
+        url = self.BASE_URL + f'models/{self.model_id}/instances'
         payload = {
             'supp_id': self.supp_id,
             'description': self.description
         }
-        resp = _format_http_response(requests.post(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.post(url, headers=HEADERS, json=payload))
         self.instance_id = self.get_instance_id()
         self.is_deployed = True
         models[self.model_id].model_instances[self.instance_id] = self
@@ -212,26 +217,26 @@ class ModelInstance:
     def update(self):
         if not self.is_deployed:
             raise Exception('This model instance has not been deployed yet. Please run `model_instance.create()` before running `model_instance.update()')
-        url = self.base_url + f'models/{self.model_id}/instances/{self.instance_id}'
+        url = self.BASE_URL + f'models/{self.model_id}/instances/{self.instance_id}'
         payload = {}
         if self.supp_id is not None:
             payload['supp_id'] = self.supp_id
         if self.description is not None:
             payload['description'] = self.description
-        resp = _format_http_response(requests.put(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.put(url, headers=HEADERS, json=payload))
         return resp['body']
 
     def delete(self):
-        url = self.base_url + f'models/{self.model_id}/instances/{self.instance_id}'
-        resp = _format_http_response(requests.delete(url, headers=self.headers))
+        url = self.BASE_URL + f'models/{self.model_id}/instances/{self.instance_id}'
+        resp = _format_http_response(requests.delete(url, headers=HEADERS))
         del models[self.instance_id]
         return resp['body']
 
     def get_response(self, human_input: list):
-        url = self.base_url + f'models/{self.model_id}/instances/{self.instance_id}/get_response'
+        url = self.BASE_URL + f'models/{self.model_id}/instances/{self.instance_id}/get_response'
         payload = {
             'human_input': human_input
         }
-        resp = _format_http_response(requests.post(url, headers=self.headers, json=payload))
+        resp = _format_http_response(requests.post(url, headers=HEADERS, json=payload))
         self.chat_history.insert(0, resp.body['interaction'])
         return resp['body']
